# CPL_CHECK_CFITSIO
#------------------
# Checks for the cfitsio library and header files.
AC_DEFUN([CPL_CHECK_CFITSIO],
[

    AC_REQUIRE([ESO_CHECK_THREADS_POSIX])
        
    cpl_cfitsio_check_version="3.270"
    cpl_cfitsio_check_header="fitsio.h"
    cpl_cfitsio_check_lib="libcfitsio.a"

    cpl_cfitsio_incdirs=""
    cpl_cfitsio_libdirs=""
    cpl_cfitsio_includes=""
    cpl_cfitsio_libraries=""

    AC_ARG_WITH(cfitsio,
                AC_HELP_STRING([--with-cfitsio],
                               [location where cfitsio is installed]),
                [
                    cpl_with_cfitsio=$withval
                ])

    AC_ARG_WITH(cfitsio-includes,
                AC_HELP_STRING([--with-cfitsio-includes],
                               [location of the cfitsio header files]),
                cpl_with_cfitsio_includes=$withval)

    AC_ARG_WITH(cfitsio-libs,
                AC_HELP_STRING([--with-cfitsio-libs],
                               [location of the cfitsio library]),
                cpl_with_cfitsio_libs=$withval)

    AC_ARG_ENABLE(cfitsio-test,
                  AC_HELP_STRING([--disable-cfitsio-test],
                                 [disables checks for the cfitsio library and headers]),
                  cpl_enable_cfitsio_test=$enableval,
                  cpl_enable_cfitsio_test=yes)


    # We need libpthread for the folloing tests

    if test -z "$LIBPTHREAD"; then
        AC_MSG_ERROR([POSIX thread library was not found on your system! Please check!])
    fi

    
    AC_MSG_CHECKING([for cfitsio])

    if test "x$cpl_enable_cfitsio_test" = xyes; then

        # Check for the cfitsio includes

        if test -z "$cpl_with_cfitsio_includes"; then
        
            if test -z "$cpl_with_cfitsio"; then
            
                # Try some known system locations
                
                cpl_cfitsio_incdirs="/opt/cfitsio/include"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/local/include"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs /usr/include"

                test -n "$CFITSIODIR" && \ 
                    cpl_cfitsio_incdirs="$CFITSIODIR/include/libcfitsio0 \
                                         $CFITSIODIR/include/cfitsio \
                                         $CFITSIODIR/include \
                                         $cpl_cfitsio_incdirs"

                test -n "$CPLDIR" && \
                    cpl_cfitsio_incdirs="$CPLDIR/include/libcfitsio0 \
                                         $CPLDIR/include/cfitsio \
                                         $CPLDIR/include \
                                         $cpl_cfitsio_incdirs"

            else

                cpl_cfitsio_incdirs="$cpl_with_cfitsio/include/libcfitsio0"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs $cpl_with_cfitsio/include/cfitsio"
                cpl_cfitsio_incdirs="$cpl_cfitsio_incdirs $cpl_with_cfitsio/include"

            fi
            
        else
            cpl_cfitsio_incdirs="$cpl_with_cfitsio_includes"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_header, $cpl_cfitsio_incdirs,
                      cpl_cfitsio_includes)


        # Check for the cfitsio library

        if test -z "$cpl_with_cfitsio_libs"; then

            if test -z "$cpl_with_cfitsio"; then
            
                # Try some known system locations

                cpl_cfitsio_libdirs="/opt/cfitsio/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/local/lib32"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs /usr/lib32"

                test -n "$CFITSIODIR" && \
                    cpl_cfitsio_libdirs="$CFITSIODIR/lib64 $CFITSIODIR/lib \
                                         $CFITSIODIR/lib32 $cpl_cfitsio_libdirs"

                test -n "$CPLDIR" && \
                    cpl_cfitsio_libdirs="$CPLDIR/lib64 $CPLDIR/lib $CPLDIR/lib32 \
                                         $cpl_cfitsio_libdirs"

            else

                cpl_cfitsio_libdirs="$cpl_with_cfitsio/lib64"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs $cpl_with_cfitsio/lib"
                cpl_cfitsio_libdirs="$cpl_cfitsio_libdirs $cpl_with_cfitsio/lib32"

            fi
            
        else
            cpl_cfitsio_libdirs="$cpl_with_cfitsio_libs"
        fi

        ESO_FIND_FILE($cpl_cfitsio_check_lib, $cpl_cfitsio_libdirs,
                      cpl_cfitsio_libraries)


        if test x"$cpl_cfitsio_includes" = xno || \
            test x"$cpl_cfitsio_libraries" = xno; then
            cpl_cfitsio_notfound=""

            if test x"$cpl_cfitsio_includes" = xno; then
                if test x"$cpl_cfitsio_libraries" = xno; then
                    cpl_cfitsio_notfound="(headers and libraries)"
                else
                    cpl_cfitsio_notfound="(headers)"
                fi
            else
                cpl_cfitsio_notfound="(libraries)"
            fi

            AC_MSG_ERROR([cfitsio $cpl_cfitsio_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_cfitsio_libraries, headers $cpl_cfitsio_includes])
        fi

        # Set up the symbols

        # Add '-lz' to the static library symbol, as distributors apparently
        # remove the libz code from the cfitsio sources.
        
        CFITSIO_INCLUDES="-I$cpl_cfitsio_includes"
        CFITSIO_LDFLAGS="-L$cpl_cfitsio_libraries"
        LIBCFITSIO="-lcfitsio"
        LIBCFITSIO_STATIC="$cpl_cfitsio_libraries/$cpl_cfitsio_check_lib -lz"
        
        # Do not add redundant libraries        
        echo $LIBS | grep -q -e '-lm' || LIBS="-lm $LIBS" 
        

        # Check whether cfitsio can be used

        AC_MSG_CHECKING([whether cfitsio can be used])
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES"
        LDFLAGS="$CFITSIO_LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC $LIBPTHREAD -lm"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <fitsio.h>
                       ]],
                       [
                       float v;
                       fits_get_version(&v);
                       ])],
                       [cpl_cfitsio_is_usable="yes"],
                       [cpl_cfitsio_is_usable="no"])

        AC_MSG_RESULT([$cpl_cfitsio_is_usable])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"

        if test x"$cpl_cfitsio_is_usable" = xno; then
            AC_MSG_ERROR([Linking with cfitsio failed! Please check architecture!])
        fi
 
        
        # Check cfitsio version

        AC_MSG_CHECKING([for a cfitsio version >= $cpl_cfitsio_check_version])
        
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES"
        LDFLAGS="$CFITSIO_LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC $LIBPTHREAD -lm"
        
        AC_RUN_IFELSE([AC_LANG_PROGRAM(
                      [[
                      #include <stdio.h>
                      #include <fitsio.h>
                      ]],
                      [
                      int vlib = 0;
                      int vmin = (int)(1000. * $cpl_cfitsio_check_version + 0.5);
                       
                      float v;

                      fits_get_version(&v);
                      vlib = (int)(v * 1000 + 0.5);
                                            
                      FILE* f = fopen("conftest.out", "w");
                      fprintf(f, "%5.3f\n", v);
                      fclose(f);
                      
                      if (vlib < vmin) {
                          return 1;
                      }

                      return 0;
                      ])],
                      [cpl_cfitsio_version="`cat conftest.out`"],
                      [
                       cpl_cfitsio_version="no";
                       cpl_cfitsio_version_found="`cat conftest.out`"
                      ])

        AC_MSG_RESULT([$cpl_cfitsio_version])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"

        if test x"$cpl_cfitsio_version" = xno; then
            AC_MSG_ERROR([Installed cfitsio ($cpl_cfitsio_version_found) is too old. Please update to version $cpl_cfitsio_check_version or newer.])
        fi
 
        
        # Check whether cfitsio has large file support
        
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES"
        LDFLAGS="$CFITSIO_LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC -lm $LIBPTHREAD"
        
        AC_MSG_CHECKING([whether cfitsio provides fits_hdu_getoff()])

        AC_COMPILE_IFELSE([AC_LANG_PROGRAM(
                          [[
                          #include <fitsio.h>
                          ]],
                          [
                          fitsfile f;
                          int sts;
                          fits_get_hduoff(&f, NULL, NULL, NULL, &sts);
                          ])],
                          [cpl_cfitsio_have_fits_get_hduoff="yes"],
                          [cpl_cfitsio_have_fits_get_hduoff="no"])

        AC_MSG_RESULT([$cpl_cfitsio_have_fits_get_hduoff])
        
        AC_MSG_CHECKING([whether cfitsio provides fits_get_hduaddrll()])

        AC_COMPILE_IFELSE([AC_LANG_PROGRAM(
                          [[
                          #include <fitsio.h>
                          ]],
                          [
                          fitsfile f;
                          int sts;
                          fits_get_hduaddrll(&f, NULL, NULL, NULL, &sts);
                          ])],
                          [cpl_cfitsio_have_fits_get_hduaddrll="yes"],
                          [cpl_cfitsio_have_fits_get_hduaddrll="no"])

        AC_MSG_RESULT([$cpl_cfitsio_have_fits_get_hduaddrll])

        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"


        # Check whether cfitsio is thread-safe
        
        AC_MSG_CHECKING([whether cfitsio requires libpthread])
        AC_LANG_PUSH(C)
        
        cpl_cfitsio_cflags_save="$CFLAGS"
        cpl_cfitsio_ldflags_save="$LDFLAGS"
        cpl_cfitsio_libs_save="$LIBS"

        CFLAGS="$CFITSIO_INCLUDES"
        LDFLAGS="$CFITSIO_LDFLAGS"
        LIBS="$LIBCFITSIO_STATIC -lm"
        
        
        cpl_cfitsio_provides_pthread="no"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <fitsio.h>
                       ]],
                       [
                       float v;
                       fits_get_version(&v);
                       ])],
                       [cpl_cfitsio_requires_pthread="no"],
                       [cpl_cfitsio_requires_pthread="undefined"])

		if test x"$cpl_cfitsio_requires_pthread" = xno; then
		
	    	# If libpthread is not required this means either cfitsio is
    		# not compiled with thread support, or the library dependencies
    		# are compiled into cfitsio.

			# Check whether shared library dependencies are present
			# Don't use pthread_mutex_init/destroy here, since glibc
			# provides these symbols too! Sigh!
		
	        AC_LINK_IFELSE([AC_LANG_PROGRAM(
		                   [[
	    	               #include <pthread.h>
    	    	           #include <fitsio.h>
        	    	       ]],
            	    	   [                	       
	                	   float v;
    	            	   pthread_mutexattr_t attrb;
    	            	   pthread_mutexattr_init(&attrb);
            	           fits_get_version(&v);
                		   pthread_mutexattr_destroy(&attrb);
	                	   ])],
	    	               [cpl_cfitsio_provides_pthread="yes"],
    	    	           [cpl_cfitsio_provides_pthread="no"])
            	           
        else
        
            LIBS="$LIBCFITSIO_STATIC -lm $LIBPTHREAD"
        
            AC_LINK_IFELSE([AC_LANG_PROGRAM(
                           [[
                           #include <fitsio.h>
                           ]],
                           [
                           float v;
                           fits_get_version(&v);
                           ])],
                           [cpl_cfitsio_requires_pthread="yes"],
                           AC_MSG_FAILURE([Cannot link with cfitsio! Please check!]))
          
        fi
                       
        AC_MSG_RESULT([$cpl_cfitsio_requires_pthread])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cfitsio_cflags_save"
        LDFLAGS="$cpl_cfitsio_ldflags_save"
        LIBS="$cpl_cfitsio_libs_save"


		AC_MSG_CHECKING([whether cfitsio was compiled with thread support])
		
		if test x"$cpl_cfitsio_requires_pthread" = xyes || \
			test x"$cpl_cfitsio_provides_pthread" = xyes; then
			cpl_cfitsio_is_thread_safe=yes
		else
			cpl_cfitsio_is_thread_safe=no
		fi
		
		AC_MSG_RESULT([$cpl_cfitsio_is_thread_safe])
		
		
        # Set compiler flags and libraries
        
        if test x"$cpl_cfitsio_have_fits_get_hduoff" = xyes || \
          test x"$cpl_cfitsio_have_fits_get_hduaddrll" = xyes; then

            CFLAGS="-D_LARGEFILE_SOURCE=1 -D_FILE_OFFSET_BITS=64 $CFLAGS"
            
            if test x"$cpl_cfitsio_have_fits_get_hduoff"; then
                AC_DEFINE([HAVE_FITS_GET_HDUOFF], [1],
                          [Define if you have the `fits_get_hduoff' function])
            else
                AC_DEFINE([HAVE_FITS_GET_HDUADDRLL],  [1],
                          [Define if you have the `fits_get_hduaddrll' function])
            fi
                    
        fi
                
        if test x"$cpl_cfitsio_requires_pthread" = xyes; then
            echo $LIBS | grep -q -e "$LIBPTHREAD" || LIBS="$LIBPTHREAD $LIBS"
        fi
        
    else
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([cfitsio checks have been disabled! This package may not build!])
        CFITSIO_INCLUDES=""
        CFITSIO_LDFLAGS=""
        LIBCFITSIO=""
        
        cpl_cfitsio_is_thread_safe="undefined"
        cpl_cfitsio_requires_pthread="undefined"
    fi

	AC_CACHE_VAL(cpl_cv_cfitsio_requires_pthread,
	             cpl_cv_cfitsio_requires_pthread=$cpl_cfitsio_requires_pthread)
	AC_CACHE_VAL(cpl_cv_cfitsio_is_thread_safe,
	             cpl_cv_cfitsio_is_thread_safe=$cpl_cfitsio_is_thread_safe)
	             
    AC_SUBST(CFITSIO_INCLUDES)
    AC_SUBST(CFITSIO_LDFLAGS)
    AC_SUBST(LIBCFITSIO)

])


