#
# CPL_CREATE_SYMBOLS(build=[])
#-----------------------------
# Sets the Makefile symbols for the CPL libraries. If an argument is
# provided the symbols are setup for building CPL, if no argument is
# given (default) the symbols are set for using the libraries
# for external package development.
AC_DEFUN([CPL_CREATE_SYMBOLS],
[

    if test -z "$1"; then
        LIBCPLCORE='-lcplcore'
        LIBCPLDRS='-lcpldrs'
        LIBCPLUI='-lcplui'
        LIBCPLDFS='-lcpldfs'
    else
        LIBCPLCORE='$(top_builddir)/cplcore/libcplcore.la'
        LIBCPLDRS='$(top_builddir)/cpldrs/libcpldrs.la'
        LIBCPLUI='$(top_builddir)/cplui/libcplui.la'
        LIBCPLDFS='$(top_builddir)/cpldfs/libcpldfs.la'
    fi

   AC_SUBST(LIBCPLCORE)
   AC_SUBST(LIBCPLDRS)
   AC_SUBST(LIBCPLUI)
   AC_SUBST(LIBCPLDFS)

])


# CPL_CHECK_LIBS
#---------------
# Checks for the CPL libraries and header files.
AC_DEFUN([CPL_CHECK_LIBS],
[

    AC_MSG_CHECKING([for CPL])

    cpl_check_cpl_header="cpl.h"
    cpl_check_cpl_lib="libcplcore.a"

    cpl_incdirs=""
    cpl_libdirs=""
    cpl_includes=""
    cpl_libraries=""

    AC_ARG_WITH(cpl,
                AC_HELP_STRING([--with-cpl],
                               [location where CPL is installed]),
                [
                    cpl_with_cpl=$withval
                ])

    AC_ARG_WITH(cpl-includes,
                AC_HELP_STRING([--with-cpl-includes],
                               [location of the CPL header files]),
                cpl_with_cpl_includes=$withval)

    AC_ARG_WITH(cpl-libs,
                AC_HELP_STRING([--with-cpl-libs],
                               [location of the CPL library]),
                cpl_with_cpl_libs=$withval)

    AC_ARG_ENABLE(cpl-test,
                  AC_HELP_STRING([--disable-cpl-test],
                                 [disables checks for the CPL library and headers]),
                  cpl_enable_cpl_test=$enableval,
                  cpl_enable_cpl_test=yes)


    if test "x$cpl_enable_cpl_test" = xyes; then
    
        # Check for the CPL includes

        if test -z "$cpl_with_cpl_includes"; then
        
            if test -z "$cpl_with_cpl"; then
            
                # Try some known system locations
                
                cpl_incdirs="/opt/cpl/include"
                cpl_incdirs="$cpl_incdirs /usr/local/include"
                cpl_incdirs="$cpl_incdirs /usr/include"

                test -n "$CPLDIR" && \
                    cpl_incdirs="$CPLDIR/include \
                                 $cpl_incdirs"

            else

                cpl_incdirs="$cpl_with_cpl/include"

            fi
            
        else
            cpl_incdirs="$cpl_with_cpl_includes"
        fi

        ESO_FIND_FILE($cpl_check_cpl_header, $cpl_incdirs, cpl_includes)


        # Check for the CPL libraries

        if test -z "$cpl_with_cpl_libs"; then

            if test -z "$cpl_with_cpl"; then
            
                # Try some known system locations

                cpl_libdirs="/opt/cpl/lib"
                cpl_libdirs="$cpl_libdirs /usr/local/lib64"
                cpl_libdirs="$cpl_libdirs /usr/local/lib"
                cpl_libdirs="$cpl_libdirs /usr/local/lib32"
                cpl_libdirs="$cpl_libdirs /usr/lib64"
                cpl_libdirs="$cpl_libdirs /usr/lib"
                cpl_libdirs="$cpl_libdirs /usr/lib32"

                test -n "$CPLDIR" && \
                    cpl_libdirs="$CPLDIR/lib64 $CPLDIR/lib $CPLDIR/lib32 \
                                 $cpl_libdirs"

            else

                cpl_libdirs="$cpl_with_cpl/lib64"
                cpl_libdirs="$cpl_libdirs $cpl_with_cpl/lib"
                cpl_libdirs="$cpl_libdirs $cpl_with_cpl/lib32"

            fi
            
        else
            cpl_libdirs="$cpl_with_cpl_libs"
        fi

        ESO_FIND_FILE($cpl_check_cpl_lib, $cpl_libdirs, cpl_libraries)


        if test x"$cpl_includes" = xno || test x"$cpl_libraries" = xno; then
            cpl_notfound=""

            if test x"$cpl_includes" = xno; then
                if test x"$cpl_libraries" = xno; then
                    cpl_notfound="(headers and libraries)"
                else
                    cpl_notfound="(headers)"
                fi
            else
                cpl_notfound="(libraries)"
            fi

            AC_MSG_ERROR([CPL $cpl_notfound was not found on your system. Please check!])
        else
            AC_MSG_RESULT([libraries $cpl_libraries, headers $cpl_includes])
        fi

    	# The libcext headers are required. By default it part of the
    	# CPL installation and it is assumed that the cext headers are
    	# present in the same location as the CPL headers
   	      	   
   	    CPL_CHECK_CEXT($cpl_includes, $cpl_libraries) 


        # Set up the symbols

        CPL_INCLUDES="-I$cpl_includes $CX_INCLUDES $CFITSIO_INCLUDES"
        CPL_LDFLAGS="-L$cpl_libraries $CX_LDFLAGS $CFITSIO_LDFLAGS"
        CPL_CREATE_SYMBOLS
 
 
        AC_MSG_CHECKING([whether CPL can be used])
        AC_LANG_PUSH(C)
        
        cpl_cflags_save="$CFLAGS"
        cpl_ldflags_save="$LDFLAGS"
        cpl_libs_save="$LIBS"

        CFLAGS="$CPL_INCLUDES"
        LDFLAGS="$CPL_LDFLAGS $LDFLAGS"
        LIBS="$LIBCPLCORE $LIBCEXT $LIBS"
        
        AC_LINK_IFELSE([AC_LANG_PROGRAM(
                       [[
                       #include <cpl.h>
                       ]],
                       [
                       cpl_init(CPL_INIT_DEFAULT);
                       ])],
                       [cpl_is_usable="yes"],
                       [cpl_is_usable="no"])

        AC_MSG_RESULT([$cpl_is_usable])
        
        AC_LANG_POP(C)
        
        CFLAGS="$cpl_cflags_save"
        LDFLAGS="$cpl_ldflags_save"
        LIBS="$cpl_libs_save"

        if test x"$cpl_is_usable" = xno; then
            AC_MSG_ERROR([Linking with CPL failed! Please check architecture!])
        fi
        
    else
    
        AC_MSG_RESULT([disabled])
        AC_MSG_WARN([CPL checks have been disabled! This package may not build!])
        CPL_INCLUDES=""
        CPL_LDFLAGS=""
        LIBCPLCORE=""
        LIBCPLDRS=""
        LIBCPLUI=""
        LIBCPLDFS=""
        
    fi

    AC_SUBST(CPL_INCLUDES)
    AC_SUBST(CPL_LDFLAGS)
    AC_SUBST(LIBCPLCORE)
    AC_SUBST(LIBCPLDRS)
    AC_SUBST(LIBCPLUI)
    AC_SUBST(LIBCPLDFS)

])
